;/*
; *****************************************************************************
; * Copyright by ams AG                                                       *
; * All rights are reserved.                                                  *
; *                                                                           *
; * IMPORTANT - PLEASE  READ  CAREFULLY  BEFORE COPYING, INSTALLING  OR USING *
; * THE SOFTWARE.                                                             *
; *                                                                           *
; * THIS  SOFTWARE  IS  PROVIDED BY  THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
; * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED *
; * TO,  THE  IMPLIED  WARRANTIES  OF  MERCHANTABILITY   AND  FITNESS  FOR  A *
; * PARTICULAR  PURPOSE  ARE  DISCLAIMED.  IN NO  EVENT  SHALL  THE COPYRIGHT *
; * OWNER  OR CONTRIBUTORS  BE LIABLE  FOR ANY DIRECT,  INDIRECT, INCIDENTAL, *
; * SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED *
; * TO, PROCUREMENT  OF SUBSTITUTE  GOODS OR  SERVICES; LOSS OF USE, DATA, OR *
; * PROFITS; OR  BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF *
; * LIABILITY, WHETHER  IN  CONTRACT, STRICT  LIABILITY, OR  TORT  (INCLUDING *
; * NEGLIGENCE  OR OTHERWISE)  ARISING  IN ANY  WAY OUT  OF THE  USE OF  THIS *
; * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              *
; *****************************************************************************
; */

;---- File    	  : GP30Y_A1.F1.11.01_GPIO_toggling.asm
;---- Version No. : 11	
;---- Build No.   : 01
;---- Created 	  : 14/12/2018  (DD/MM/YYYY)
;---- last change : 14/12/2018
;---- Author	  : Matthias Hainz
;---- Updated by  : Matthias Hainz
;---- Company  	  : ams Sensors Germany GmbH
;---- Project  	  : GP30Y
;----
;---- Contents 	  : Simple Firmware for toggling GPIO output 


;============================== Include files ========================================

#include "GP30Y_REG_A1.2.h"			; Definition of Register addresses, bit positions and register actions
#include "GP30Y_A1.D2.11.04.h"		; Definition of constants, memory organisation etc.
#include "GP30Y_ROM_A1.common.h"	; Definition of ROM routine start addresses

;============================== CONSTANTS / DEFINITIONS ==============================
; The following is used for debugging
CONST NUMBER_OF_RUNS			0x40			; Counts the runs (for debugging)
CONST DEBUG_ADDR_01				0x41			; Address (for debugging)
CONST DEBUG_CONTENT_01			0x101010		; Specified content (for debugging)
CONST DEBUG_ADDR_02				0x42			; Address (for debugging)
CONST DEBUG_CONTENT_02			0x202020		; Specified content (for debugging)

CONST REV_ADDRESS_VAR			3996			; location of Version number in FW hex code with empty chip

CONST FW_ROMVERSION_REV			0xA1			; ROM version
CONST FW_VERSION_NUM			0xF10000		; F denots ams example code
CONST FW_VERSION_MAJ			0x001000		; Major version 1
CONST FW_VERSION_MIN			0x000100		; Minor version 1
CONST FW_VERSION_BLD			0x000001		; Current build 01
CONST FW_VERSION				FW_VERSION_NUM + FW_VERSION_MAJ + FW_VERSION_MIN + FW_VERSION_BLD  ;

;============================== Firmware Main Body ===================================
; The following org command is needed at the start of the code
org 0

;=====================================================================================
;=================== Check of CPU Requests - Main Firmware routine 
;=====================================================================================
; This routine checks the status flags for activity requests and calls the according routines	
; Inputs 		:	Status flags in SRR_FEP_STF, SRR_ERR_FLG and SHR_CPU_REQ
; Output 		:	all processing done and status flags cleared
; (total)unused	: (all used)
; Temporary RAM : -	
; Permanent RAM : 	
; Routines used :  
;=====================================================================================

MK_CPU_REQ:

; ------------------- Counts the runs (for debugging)
	ramadr NUMBER_OF_RUNS
	incr		r
; -------------------

;**************************************************************************
ramadr 		SHR_CPU_REQ					;-- Set RAM Address to SHR_CPU_REQ
skipBitC	r,	BNR_FWI, 1				;-- Check Firmware Init Flag
	jsub 		MK_FW_INIT				;-- Jump to Firmware Init if it is first start
;**************************************************************************

;**************************************************************************
	ramadr 	SRR_ERR_FLG					;-- Set RAM Address to SRR_ERR_FLG
	getflag	r							;-- Set the signum and zero flag according to the addressed register
	skipEQ 	1							;-- Skip one commands if SRR_ERR_FLG is equal to zero. 
		jsub 	MK_OWN_EH				;-- Jump to User Error Handling Subroutine
;**************************************************************************

;**************************************************************************
	ramadr 		SRR_FEP_STF				;-- Set RAM Address to SRR_FEP_STF	
	skipBitC 	r, BNR_TOF_UPD, 1		;-- Check US_TOF_UPD Flag
		jsub 		MK_OWN_CODE			;-- Jump to User Code
;**************************************************************************

MK_STOP:

	ramadr		SHR_CPU_REQ				; Set RAM Address to SHR_CPU_REQ
	clear		r						; Clear SHR_CPU_REQ
	
	clrwdt								; Clearing watchdog
stop

;#####################################################################################
;      End of Main Program MK_CPU_REQ
;#####################################################################################

;=====================================================================================
;==================== place your own code here for error handling 
;=====================================================================================
; The subroutine is not active yet and just clears the error flag register
; Inputs 		:	-
; Output 		:	-
; (total)unused	: (all used)
; Temporary RAM : -	
; Permanent RAM : -	
; Routines used : - 
;=====================================================================================

MK_OWN_EH:

; ------------------- Writes into RAM (for debugging)
	move y, DEBUG_CONTENT_01
	ramadr DEBUG_ADDR_01
	move r, y
; -------------------

	;############ Place your code here for error handling ############
	nop								;-- 
;	nop								;-- 
;	nop								;-- 
;	nop								;-- 
	;#################################################################

MK_OWN_EH_END:

	ramadr		SHR_EXC					; Set RAM Address to SHR_EXC
	bitset		r,	BNR_EF_CLR			; clear error flag register

	jsubret

;#####################################################################################
;      End of Subroutine MK_OWN_EH
;#####################################################################################

;=====================================================================================     
;=================== place your own code here for post processing 
;=====================================================================================
; The subroutine is simple firmware for toggling GPIO output
; Inputs 		:	-
; Output 		:	-
; (total)unused	: (all used)
; Temporary RAM : -	
; Permanent RAM : -	
; Routines used : - 
;=====================================================================================

MK_OWN_CODE:

; ------------------- Writes into RAM (for debugging)
	move y, DEBUG_CONTENT_02
	ramadr DEBUG_ADDR_02
	move r, y
; -------------------

; ~400us after TOF measurement (including both directions)
	ramadr		SHR_GPO				; set and reset takes about ~200ns
	bitset		r, BNR_GPO4_OUT
	bitclr		r, BNR_GPO4_OUT

	;bitinv		r, BNR_GPO4_OUT		; toggling between the TOF measurements

MK_OWN_END:

	jsubret

;#####################################################################################
;      End of Subroutine MK_OWN_CODE
;#####################################################################################

;=====================================================================================
;==================== place your own code here for initialization 
;=====================================================================================
; The subroutine is to configure a fixed value for the ZCD_FHL_U and  ZCD_FHL_D bits.
; These First Hit Level (FHL) values are specific to the hardware.
; Inputs 		:	-
; Output 		:	-
; (total)unused	: (all used)
; Temporary RAM : -	
; Permanent RAM : -	
; Routines used : - 
;=====================================================================================

MK_FW_INIT:

	jsub		ROM_USER_RAM_INIT		; Initialising all USER RAM cells to 0

	ramadr		FWD_R1_FHL_VALUE		;---- Value for the first hit level
	move		x, r

	ramadr		SHR_ZCD_FHL_U			; Setting the same FHL for Up and Down
	move		r, x
	ramadr		SHR_ZCD_FHL_D
	move		r, x
	
MK_FW_INIT_END:

	ramadr		SHR_CPU_REQ				; Set RAM Address to SHR_CPU_REQ
    bitclr		r,	BNR_FWI				; Clear Firmware Init Flag
	
	jsubret								; Jump Back

;#####################################################################################
;      End of Subroutine MK_FW_INIT
;#####################################################################################

;stop

;==================================== End Of Firmware ================================


;=====================================================================================
;==== Write Version number at the end of the writable hex code in empty chip case 
;=====================================================================================
	
	org 	REV_ADDRESS_VAR            	; update of vA1.F1.12.02
	equal1	FW_ROMVERSION_REV
	equal	FW_VERSION					; Defined at the beginning of this file
	
;=====================================================================================
;==================== Fill file with zeros for locked memory part 
;=====================================================================================

	org 4095
	nop
